<?php
/**
 * Class EM_Dashboard_OptionsPage
 *
 * Adds options pages for the plugin
 *
 * @since 1.0.0
 */
class EM_Dashboard_OptionsPage extends EM_Dashboard_Options {

	/**
	 * Constructor, load parent constructor
	 */
	public function __construct() {
		parent::__construct();

		add_action( 'admin_init', array( $this, 'register_fields' ) );

		if ( is_multisite() ) {
			add_action( 'network_admin_menu', array( $this, 'add_menu_link' ) );
			add_action( 'admin_init', array( $this, 'maybe_save_multisite' ) );
		} else {
			add_action( 'admin_menu', array( $this, 'add_menu_link' ) );
		}
	}

	/**
	 * Add Option fields to /settings_or_options-general.php?page=em_dashboard_settings
	 *
	 * @since 1.0.0
	 * @return void
	 */
	public function register_fields() {

		/**
		 * Start General Settings
		 *
		 * @since 1.0.0
		 */
		register_setting( 'em_dashboard_g_settings', 'em_dashboard_em_styles', 				array( $this, 's_boolean' ) ); //default false
		register_setting( 'em_dashboard_g_settings', 'em_dashboard_em_fonts', 				array( $this, 's_boolean' ) ); //default true
		register_setting( 'em_dashboard_g_settings', 'em_dashboard_color_scheme', 			array( $this, 's_colorscheme' ) ); //default light
		register_setting( 'em_dashboard_g_settings', 'em_dashboard_color_picker', 			array( $this, 's_boolean' ) ); //Default true
		register_setting( 'em_dashboard_g_settings', 'em_dashboard_load_easy', 				array( $this, 's_boolean' ) ); // Default true
		register_setting( 'em_dashboard_g_settings', 'em_dashboard_easy_mode', 				array( $this, 's_default_easy' ) ); // Default 'off'
		register_setting( 'em_dashboard_g_settings', 'em_dashboard_force_easy', 			array( $this, 's_boolean' ) ); //default false
		register_setting( 'em_dashboard_g_settings', 'em_dashboard_force_easy_capability', 	array( $this, 's_capability' ) ); //default install_plugins

		add_settings_section(
			'em_dashboard_general_settings',			// id
			__('General Settings', 'em-dashboard'),		// title
			array( $this, 'general_settings_fields'),	// callback
			'em_dashboard_g_settings'					// page
		);

		/**
		 * General settings section tab
		 *
		 * Doing it right: the WP way. Boring. Good.
		 */
		add_settings_field (
			'em_dashboard_em_styles', 							// id
			__( 'EM Dashboard Styles', 'em-dashboard' ),		// title
			array( $this, 'checkbox' ),				 			// callback
			'em_dashboard_g_settings',		 	 				// page
			'em_dashboard_general_settings',				 	// section
			array( 												// args
				'name' 		=> 'em_dashboard_em_styles',
				'page'		=> 'em_dashboard_g_settings',
				'default' 	=> 'em_styles',
				'labelinfo'	=> __( 'Load the EM Dashboard theme.', 'em-dashboard' ),
				'hoverinfo'	=> __( 'Experimental flat soft WP theme (this also removes the color picker)', 'em-dashboard' ),
				'hoverinfosite'	=> '',
				'hoverinfonetwork' => '',
				'class'		=> 'alternate',
			)
		);
		add_settings_field (
			'em_dashboard_em_fonts', 						// id
			__( 'EM Dashboard Fonts', 'em-dashboard' ), 	// title
			array( $this, 'checkbox' ),	 					// callback
			'em_dashboard_g_settings',		 	 			// page
			'em_dashboard_general_settings',				// section
			array( 											// args
				'name' 		=> 'em_dashboard_em_fonts',
				'page'		=> 'em_dashboard_g_settings',
				'default' 	=> 'em_fonts',
				'labelinfo'	=> __( 'Load the Merriweather Google Font that goes with the EM Dashboard theme.', 'em-dashboard' ),
				'hoverinfo'	=> __( 'This option only has effect if the EM Dashboard Styles are also active', 'em-dashboard' ),
				'hoverinfosite'	=> '',
				'hoverinfonetwork' => '',
				'class'		=> 'alternate',
			)
		);
		add_settings_field (
			'em_dashboard_color_scheme', 					// id
			__( 'Default Color Scheme', 'em-dashboard' ), 	// title
			array( $this, 'dropdown'),	 					// callback
			'em_dashboard_g_settings',		 		 		// page
			'em_dashboard_general_settings',				// section
			array( 											// args
				'name' 		=> 'em_dashboard_color_scheme',
				'page'		=> 'em_dashboard_g_settings',
				'default' 	=> 'color_scheme',
				'labelinfo'	=> __( 'Set the default theme for newly registered users.', 'em-dashboard' ),
				'hoverinfo'	=> __( 'This does not affect currently registered users.', 'em-dashboard' ),
				'hoverinfosite'	=> '',
				'hoverinfonetwork' => '',
				'class'		=> '',
			)
		);
		add_settings_field (
			'em_dashboard_color_picker', 				// id
			__( 'Enable Color Picker', 'em-dashboard' ), // title
			array( $this, 'checkbox' ),					// callback
			'em_dashboard_g_settings',			 	 	// page
			'em_dashboard_general_settings',			// section
			array( 										// args
				'name' 		=> 'em_dashboard_color_picker',
				'page'		=> 'em_dashboard_g_settings',
				'default' 	=> 'color_picker',
				'labelinfo'	=> __( 'When this option is on, the color picker is visible in the user profile page.', 'em-dashboard' ),
				'hoverinfo'	=> '',
				'hoverinfosite'	=> __( "This option is set to on if 'Enable EM Dashboard Styles' is on. This has no effect on the admin", 'em-dashboard' ),
				'hoverinfonetwork'	=> __( "This option is set to on if 'Enable EM Dashboard Styles' is on. This has no effect on the super-admin", 'em-dashboard' ),
				'class'		=> '',
			)
		);
		add_settings_field (
			'em_dashboard_load_easy', 							// id
			__( 'Enable Easy Mode Button', 'em-dashboard' ),   	// title
			array( $this, 'checkbox'), 							// callback
			'em_dashboard_g_settings',		 	 				// page
			'em_dashboard_general_settings',					// section
			array( 												// args
				'name' 		=> 'em_dashboard_load_easy',
				'page'		=> 'em_dashboard_g_settings',
				'default' 	=> 'load_easy',
				'labelinfo'	=> __( 'Enable the easy mode button.', 'em-dashboard' ),
				'hoverinfo'	=> '',
				'hoverinfosite'	=> __( 'The button on the bottom left.', 'em-dashboard' ),
				'hoverinfonetwork'	=> __( "The button on the bottom left. This doesn't have effect in the network admin screen", 'em-dashboard' ),
				'class'		=> 'alternate',
			)
		);
		add_settings_field (
			'em_dashboard_easy_mode', 						// id
			__( 'Default Easy Mode Setting', 'em-dashboard' ), // title
			array( $this, 'dropdown'), 						// callback
			'em_dashboard_g_settings',		 	 			// page
			'em_dashboard_general_settings',				// section
			array( 											// args
				'name' 		=> 'em_dashboard_easy_mode',
				'page'		=> 'em_dashboard_g_settings',
				'default' 	=> 'easy_mode',
				'labelinfo'	=> __( 'Turn the button on the bottom left on or off for new users.', 'em-dashboard' ),
				'hoverinfo'	=> __( "This option only has effect on users who haven't touched it yet", 'em-dashboard' ),
				'hoverinfosite'	=> '',
				'hoverinfonetwork' => '',
				'class'		=> 'alternate',
			)
		);
		add_settings_field (
			'em_dashboard_force_easy', 					// id
			__( 'Force Easy Mode', 'em-dashboard' ), 		// title
			array( $this, 'checkbox'), 					// callback
			'em_dashboard_g_settings',		 			// page
			'em_dashboard_general_settings',			// section
			array( 										// args
				'name' 		=> 'em_dashboard_force_easy',
				'page'		=> 'em_dashboard_g_settings',
				'default' 	=> array( 'force_easy', '0'),
				'labelinfo'	=> __( "Force Easy Mode for everyone except for the set Capability below.", 'em-dashboard' ),
				'hoverinfo'	=> __( 'This option also removes the button on the bottom left except for the minimum capability.', 'em-dashboard' ),
				'hoverinfosite'	=> '',
				'hoverinfonetwork' => '',
				'class'		=> '',
			)
		);
		add_settings_field (
			'em_dashboard_force_easy_capability', 					// id
			__( 'Minimum Non-Forced Capability', 'em-dashboard' ),	// title
			array( $this, 'dropdown'),								// callback
			'em_dashboard_g_settings',				 				// page
			'em_dashboard_general_settings',			 			// section
			array( 													// args
				'name' 		=> 'em_dashboard_force_easy_capability',
				'page'		=> 'em_dashboard_g_settings',
				'default' 	=> array( 'force_easy', '1' ),
				'labelinfo'	=> __( "The minimum role that doesn't get Easy Mode forced and sees the Easy Mode Button at all times regardless of other settings.", 'em-dashboard' ),
				'hoverinfo'	=> __( "Although very unlikely, if you get stuck in Easy Mode, please see the plugin page's 'Other Nodes'.", 'em-dashboard' ),
				'hoverinfosite'	=> '',
				'hoverinfonetwork' => '',
				'class'		=> '',
			)
		);
	}

	/**
	 * Adds menu links under "settings" in the wp-admin dashboard
	 *
	 * @since 1.0.0
	 * @return void
	 */
	public function add_menu_link() {
		$parent_slug = is_multisite() ? 'settings.php' : 'options-general.php';

		$pagetitle = __( 'EM Dashboard Settings', 'em-dashboard' );
		$menutitle = 'EM Dashboard';
		$capability = 'install_plugins';
		$menu_slug = 'em_dashboard_settings';

		$callback = array( $this, 'menu_page' );

		add_submenu_page( $parent_slug, $pagetitle, $menutitle, $capability, $menu_slug, $callback );
	}

	/**
	 * Initialize the tabbed menu page
	 *
	 * @since 1.0.0
	 * @return bool false if not admin || string menu page if admin
	 */
	public function menu_page() {

		$title = __( 'EM Dashboard Settings', 'em-dashboard' );

		// WordPress 4.3.0 has new accessibility standards for screen readers (mainly h1/h2 from what I've read)
		$wp430 = $this->wp_version( '4.3.0', '>=' );

		?>
		<div class="wrap">

			<?php if ( $wp430 ) : ?>
				<h1><?php echo $title; ?></h1>
			<?php else : ?>
			    <h2><?php echo $title; ?></h2>
			<?php endif; ?>

			<?php // Multisite only header. A little more elegant than "Settings saved." :)
			if ( isset( $_GET['updated'] ) ) { ?>
				<div id="message" class="updated notice is-dismissible">
					<p>
						<?php _e( 'Your settings have been saved', 'em-dashboard' ) ?>
					</p>
				</div>
			<?php
			}

			if ( is_multisite() ) {
				$action = 'settings.php?page=em_dashboard_settings';
			} else {
				$action = 'options.php';
			}
			?>
			<form method="post" action="<?php echo esc_attr( $action ); ?>">
				<?php
					if ( is_multisite() ) {
						// Hack?
						wp_nonce_field( '_update_em_multisite', '_em_multisite_nonce' );
					}
					do_settings_sections( 'em_dashboard_g_settings' );
				?>
				<hr>
				<?php submit_button(); ?>
			</form>
		</div>
		<?php
	}

	public function maybe_save_multisite() {

		if ( ! isset( $_POST['submit'] ) ) return;

		if ( ! current_user_can( 'install_plugins' ) ) return;

		check_admin_referer( '_update_em_multisite', '_em_multisite_nonce' );

		// var_dump( $_POST, wp_get_referer(), wp_verify_nonce( $_POST['_em_multisite_nonce'], '_update_em_multisite' ) );exit;

		// em_dashboard_g_settings
		$settings = array_fill_keys( array(
			'em_dashboard_em_styles',
			'em_dashboard_em_fonts',
			'em_dashboard_color_picker',
			'em_dashboard_color_scheme',
			'em_dashboard_load_easy',
			'em_dashboard_easy_mode',
			'em_dashboard_force_easy',
			'em_dashboard_force_easy_capability',
		), '' );

		// Although duplicate $_POST values are sent with the checkboxes, I'm not sure why only a single value is parsed.
		// We'll see if someone returns with a bug report on odd settings. If so, I'll try to uniqueify the array.
		foreach ( (array) $_POST as $key => $value ) {
			if ( array_key_exists( $key, $settings ) ) {
				update_site_option( $key, $value );
			}
		}

		// $url = add_query_arg( 'page', 'em_dashboard_settings', network_admin_url( 'settings.php' ) );
		$url = add_query_arg( array( 'updated' => 'true' ), wp_get_referer() );
		wp_safe_redirect( esc_url_raw( $url ) );
		exit;
	}

	/**
	 * Register general settings and add welcome field.
	 *
	 * @since 1.0.0
	 */
	public function general_settings_fields() {

		settings_fields('em_dashboard_g_settings');

		if ( is_multisite() ) {
			$welcome = __( 'The settings on this page affect every user and site on your network.', 'em-dashboard' );
		} else {
			$welcome = __( 'The settings on this page affect every user on your website', 'em-dashboard' );
		}

		?><p style="display:inline-block;width:auto;padding:10px;margin:3px 0;border:1px solid #aaa;"><?php echo esc_html( $welcome ); ?></p><?php
	}

	/**
	 * A checkbox setting field
	 *
	 * @param array $args the settings argument
	 * @return string the settings field
	 *
	 * @since 1.0.0
	 */
	public function checkbox( $args = array() ) {

		$name	 		= $args['name'];
		$page 			= $args['page'];
		$default 		= $args['default'];
		$info 			= $args['labelinfo'];
		$label_hover	= $args['hoverinfo'];
		$hover_site 	= $args['hoverinfosite'];
		$hover_network	= $args['hoverinfonetwork'];

		$option = $this->option();

		if ( is_array( $default ) ) {
			$option_name = $default[0];

			$default = $default[1];
			$option  = $option[$option_name];
		}
		$the_option = get_site_option( $name, $option[$default] );

		if ( empty( $label_hover ) ) {
			if ( is_multisite() ) {
				$label_hover = $hover_network;
			} else {
				$label_hover = $hover_site;
			}
		}

		?><label>
			<input type="hidden" name="<?php echo esc_attr( $name ); ?>" value="0">
			<input type="checkbox" name="<?php echo esc_attr( $name ); ?>" id="<?php echo esc_attr( $name ); ?>" value="1" <?php checked( $the_option ) ?> />
			<em><?php echo esc_html( $info ); ?></em>
			<?php
			if ( !empty( $label_hover ) ) {
				 ?><span style="text-decoration:underline" title="<?php echo esc_attr( $label_hover ); ?>">[?]</span><?php
			}
		?></label><?php

	}

	/**
	 * A dropdown setting field
	 *
	 * Set the option values here based on the selected option.
	 *
	 * @param array $args the settings argument
	 * @return string the settings field
	 *
	 * @todo maybe external filter for more options
	 *
	 * @since 1.0.0
	 */
	public function dropdown( $args = array() ) {

		$name	 		= $args['name'];
		$page 			= $args['page'];
		$default 		= $args['default'];
		$info		 	= $args['labelinfo'];
		$label_hover 	= $args['hoverinfo'];
		$hover_site 	= $args['hoverinfosite'];
		$hover_network	= $args['hoverinfonetwork'];

		$option = $this->option();

		if ( is_array( $default ) ) {
			$option_name = $default[0];

			$default = $default[1];
			$option = $option[$option_name];
		}

		$the_option = get_site_option( $name, $option[$default] );

		$options = array();
		$optionsi18n = array();

		/**
		 * Start Options
		 *
		 * @since 1.0.0
		 */
		// Start color_scheme
		if ( $name === 'em_dashboard_color_scheme' ) {
			global $_wp_admin_css_colors;

			ksort( $_wp_admin_css_colors );

			if ( isset( $_wp_admin_css_colors ) ) {
				foreach ( $_wp_admin_css_colors as $color => $color_info ) { // Too tired/lazy at this point to check if the => $color_info is needed.
					$options[] = $color;
					$optionsi18n[] = $color_info->name;
				}
			// Fallback
			} else {
				$options = array( 'default', 'light', 'blue', 'midnight', 'sunrise', 'ectoplasm', 'ocean', 'coffee' );
				$optionsi18n = array( __('Default'), __('Light'), __('Blue'), __('Midnight'), __('Sunrise'), __('Ectoplasm'), __('Ocean'), __('Coffee') );
			}
		// Start easy_mode
		} else if ( $name === 'em_dashboard_easy_mode' ) {
			$options = array( 'on', 'off');
			$optionsi18n = array(
				__( 'On', 'em-dashboard' ),
				__( 'Off', 'em-dashboard' ),
			);
		// Start Force Easy Capabilities
		} else if ( $name === 'em_dashboard_force_easy_capability') {
			//* Show only roles the current user is/can edit. Prevents getting stuck.
			$roles = get_editable_roles(); // this crashes outside the WP admin area. No problem.
			foreach ( $roles as $role_name => $role_info ) {
				foreach ( $role_info['capabilities'] as $capability => $bool ) {
					$capabilities[] = $capability;
				}
			}

			// Extremely elegant to repeat. :) But it's faster than array merge.
			// Not so much faster for the translators, however.
			if ( is_multisite() ) {
				$caps = array(
					'install_plugins' 			=> __( 'Super Administrator', 'em-dashboard' ),
					'manage_options' 			=> __( 'Administrator', 'em-dashboard' ),
					'publish_pages' 			=> __( 'Editor', 'em-dashboard' ),
					'publish_posts' 			=> __( 'Author', 'em-dashboard' ),
					'edit_posts' 				=> __( 'Contributor', 'em-dashboard' ),
					'read' 						=> __( 'Subscriber', 'em-dashboard' ),
				);
			} else {
				$caps = array(
					'manage_options' 			=> __( 'Administrator', 'em-dashboard' ),
					'publish_pages' 			=> __( 'Editor', 'em-dashboard' ),
					'publish_posts' 			=> __( 'Author', 'em-dashboard' ),
					'edit_posts' 				=> __( 'Contributor', 'em-dashboard' ),
					'read' 						=> __( 'Subscriber', 'em-dashboard' ),
				);
			}

			foreach ( $caps as $cap => $capi18n ) {
				if ( in_array( $cap, $capabilities) ) {
					$options[] = $cap;
					$optionsi18n[] = $capi18n;
				}
			}
		} else {
			// Option isn't parsed, bail. Doesn't really need translation.
			?><em>Options failed to load. Please report to the plugin author.</em><?php
			return;
		}

		?>
		<select name="<?php echo esc_attr( $name ); ?>" value="<?php echo esc_attr( $the_option ); ?>">
			<?php foreach ( array_combine( $options, $optionsi18n ) as $sel_option => $sel_option_i18n ) {
					$selected = ( $the_option == $sel_option ) ? 'selected' : '';
					// Fall back to option name is no i18n is set.
					$sel_option_i18n = ! empty( $sel_option_i18n ) ? $sel_option_i18n : $sel_option;

					echo '<option value="' . esc_attr( $sel_option ) . '" ' . $selected . '>' . esc_html( $sel_option_i18n ) . '</option>';
			}
			?>
		</select>

		<label for="<?php echo esc_attr( $name ); ?>"><em><?php echo esc_html( $info ); ?> </em>
		<?php
		if ( !empty( $label_hover ) ) {
			?><span style="text-decoration:underline" title="<?php echo esc_attr( $label_hover ); ?>">[?]</span><?php
		}
		?></label><?php
	}

	/**
	 * Sanitize Checked
	 *
	 * @since 1.0.0
	 *
	 * @param bool/string $value the option
	 * @return string 1 or 0 the sanitized option
	 *
	 * Unused for now
	 */
	public function s_checked( $value, $default = '' ) {

		if ( $value === 'on' ) {
			return '1';
		} elseif ( $value == 'off') {
			return '0';
		}

		return $default;
	}

	/**
	 * Sanitize boolean
	 *
	 * @since 1.0.0
	 *
	 * @param bool/string $value the option
	 * @return string 1 or 0 the sanitized option
	 *
	 */
	public function s_boolean( $value, $default = '' ) {

		if ( $value === '1' || $value === 'on' ) {
			return true;
		} elseif ( $value === '0' || $value === 'off' ) {
			return false;
		} elseif ( is_bool( $value ) ) {
			return $value;
		}

		return $default;
	}

	/**
	 * Sanitize color_scheme
	 *
	 * @since 1.0.0
	 *
	 * @param string $value the color scheme
	 * @return string the sanitized option
	 */
	public function s_colorscheme( $value, $default = '' ) {
		global $_wp_admin_css_colors;

		ksort( $_wp_admin_css_colors );

		if ( isset( $_wp_admin_css_colors ) ) {
			foreach ( $_wp_admin_css_colors as $color => $color_info ) {
					$array[] = $color;
			}
			if ( in_array( $value, $array, true ) ) {
				return sanitize_key( $value );
			}
		// Fallback
		} else {
			$color[] = array('default', 'light', 'blue', 'midnight', 'sunrise', 'ectoplasm', 'ocean', 'coffee');
			if ( in_array( $value, $color, true ) ) {
				return esc_attr($value);
			}
		}

		// Back to default if sanitation failed
		return $default;
	}

	/**
	 * Sanitize Default Easy setting (on/off)
	 *
	 * @since 1.0.0
	 */
	public function s_default_easy( $value, $default = '' ) {

		if ( $value === 'on' || $value === 'off' ) {
			return $value;
		} elseif ( $value ) {
			return 'on';
		}

		return 'off';
	}

	/**
	 * Sanitize Capability
	 *
	 * @since 1.0.0
	 * @todo fetch caps from main options array. This way we can determine default too.
	 */
	public function s_capability( $value, $default = '' ) {

		$caps = array( 'install_plugins', 'manage_options', 'publish_pages', 'publish_posts', 'edit_posts', 'read' );

		if ( in_array( $value, $caps, true ) ) {
			return $value;
		}

		return $default;
	}
}
